"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Crumbl_1 = require("../../core/Crumbl");
const __1 = require("..");
const crypto_1 = require("../../crypto");
const Uncrumb_1 = require("../../Decrypter/Uncrumb");
const Uncrumbl_1 = require("../../core/Uncrumbl");
class BrowserWorker {
    constructor(p) {
        this.mode = p.mode;
        this.data = p.data;
        this.verificationHash = p.verificationHash;
        this.htmlElement = p.htmlElement;
        // Check data
        if (this.data.length === 0) {
            const msg = 'no data to use';
            throw new Error(msg);
        }
    }
    /**
       * Create a new crumbl using the passed credentials
       *
       * @param owners    A list of data owners
       * @param trustees  A list of third-party signers
       */
    create(owners, trustees) {
        return __awaiter(this, void 0, void 0, function* () {
            // Check mode coherence
            if (this.mode !== __1.CREATION) {
                return Promise.reject(new Error(`invalid mode: ${this.mode}`));
            }
            // Build returned result
            let result = '';
            const crumbl = new Crumbl_1.Crumbl(this.data[0], crypto_1.DEFAULT_CRYPTO_HASH_ENGINE, owners, trustees);
            if (this.htmlElement === undefined) {
                result = yield crumbl.process();
            }
            else {
                result = yield crumbl.toHTML(this.htmlElement);
            }
            // Optionally check verification hash
            if (result !== '' && this.verificationHash !== '' && this.verificationHash !== undefined) {
                if (!result.startsWith(this.verificationHash)) {
                    return Promise.reject(new Error('verification hash is not coherent with data source'));
                }
            }
            return result;
        });
    }
    /**
       * Extract either the partial uncrumbs or the fully-decrypted value for the passed signer
       *
       * @param user      The signer credentials
       * @param isOwner   Pass `true` if the user is a data owner, `false` otherwise
       */
    extract(user, isOwner) {
        return __awaiter(this, void 0, void 0, function* () {
            // Check mode coherence
            if (this.mode !== __1.EXTRACTION) {
                return Promise.reject(`invalid mode:  ${this.mode}`);
            }
            // Get the partial uncrumbs
            const uncrumbs = new Array();
            if (this.data.length > 1) {
                for (let i = 1; i < this.data.length; i++) {
                    const parts = this.data[i].split('.', 2);
                    if (parts[1] !== Crumbl_1.VERSION) {
                        return Promise.reject(`wrong version for uncrumb: ${this.data[i]}`);
                    }
                    const us = parts[0].substr(crypto_1.DEFAULT_HASH_LENGTH);
                    const uncs = us.split(Uncrumb_1.PARTIAL_PREFIX);
                    for (let i = 0; i < uncs.length; i++) {
                        const unc = uncs[i];
                        if (unc !== '') {
                            const uncrumb = Uncrumb_1.toUncrumb(unc);
                            uncrumbs.push(uncrumb);
                        }
                    }
                }
            }
            // Build returned result
            let result = '';
            if (this.verificationHash === undefined)
                throw new Error('verification hash cannot be empty');
            const uncrumbl = new Uncrumbl_1.Uncrumbl(this.data[0], uncrumbs, this.verificationHash, user, isOwner);
            if (this.htmlElement === undefined) {
                result = (yield uncrumbl.process()).toString();
            }
            else {
                result = (yield uncrumbl.toHTML(this.htmlElement)).toString();
            }
            // Check verification hash
            if (this.verificationHash === '') {
                return Promise.reject(new Error('verification hash is missing'));
            }
            else if (isOwner) {
                const hashedResult = yield crypto_1.hash(result); // TODO add hashEngine in worker and pass it here?
                if (hashedResult !== this.verificationHash) {
                    return Promise.reject(new Error('verification hash is not coherent with uncrumbled data'));
                }
            }
            return result;
        });
    }
}
exports.BrowserWorker = BrowserWorker;
