"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DEFAULT_CRYPTO_HASH_ENGINE = 'sha-256';
exports.DEFAULT_HASH_LENGTH = 64;
exports.ECIES_ALGORITHM = 'ecies';
exports.RSA_ALGORITHM = 'rsa';
const authorizedAlgorithms = [exports.ECIES_ALGORITHM, exports.RSA_ALGORITHM];
/**
 * Test whether the passed algorithm is compliant with the current system
 *
 * @param name the algorithm code name to test
 */
exports.existsAlgorithm = (name) => {
    return authorizedAlgorithms.includes(name);
};
/**
 * Hash the passed string using SHA-256 hash algorithm (as of the latest version of the Crumbl&trade;)
 *
 * @param string the data to hash
 * @param hashEngine the name of the hash engine (default: `sha-256`)
 * @returns the hexadecimal representation of the hashed data
 */
exports.hash = (data, hashEngine = exports.DEFAULT_CRYPTO_HASH_ENGINE) => new Promise((resolve, reject) => {
    if (hashEngine !== exports.DEFAULT_CRYPTO_HASH_ENGINE)
        reject(new Error('invalid hash engine'));
    resolve(crypto.subtle.digest(hashEngine, Buffer.from(data)).then(digested => Buffer.from(digested).toString('hex')));
});
